#ifndef ADOUT_H
#define ADOUT_H

#include "typedef.h"

/*{{{  << COMMENT -- implicit type rules;  form of 'ADAPT'-produced IDs >>*/
#if 0

/*{{{  ID prefixing rules*/
ID prefixing rules
------------------

On output, every ID *except* for an intrinsic name is preceded
by a prefix "%c_",  where the letter '%c' depends on the data type
and the origin of the ID as follows:

	-----------------------------------------
			|  %c -- (see below)	|
	data type	|   1	|   2	|   3	|
	----------------|-------|-------|-------|
	INTEGER		|   I       J       K	|
	REAL		|   R       S       T	|
	DP		|   D       E       F	|
	COMPLEX		|   X       Y       Z	|
	CHAR		|   A       B       C	|
	LOGICAL		|   L       M       N	|
	NONE / untyped /|   O       P       Q	|
	  other		|			|
	-----------------------------------------

The column numbers mean the following:

1 -- all user IDs except for 2.
2 -- array-valued dummy arguments that must be remapped;
3 -- all internally-generated IDs.
/*}}}*/

/*{{{  Format of 'ADAPT'-produced identifiers*/
Format of 'ADAPT'-produced identifiers
--------------------------------------

INTEGER ID's
~~~~~~~~~~~~
K_%d	-- DO-variable for ADLIB-produced DO-loop over the '%d'th
	   regular section dimension of an array assignment.

K_C%d	-- as for 'K_%d', and used in conjunction with it when
	   the dimension has 'CYCLIC' distribution.

K_H%s	-- } handle to 'adlib' data structure for named processor
K_H%d	-- } arrangement (%s) or default proc arrangement (%d -- "$p%d")

K_M%s	-- mapvec for '%s'
K_M%d	-- subscript in 'K_TMP ()' of start of mapvec for temporary #'%d'

K_N%s	-- dummy argument mapvec for dummy argument array '%s' (which
	   will be remapped!).

K_P%s	-- } vector containing coordinates of local processor in a named
K_P%d	-- } proc arrangement (%s) or default proc arrangement ("$p%d")

K_T%d	--  subscript in 'K_TMP ()' of start of temporary array #'%d'

K_TMP	-- global array for allocating temporary storage

K_V%d	-- scalar temporary variable

LOGICAL ID's
~~~~~~~~~~~~
N_P%s	-- extra dummy argument introduced for an optional dummy arg '%s';
	   TRUE if corresponding actual arg is present, otherwise FALSE.
/*}}}*/

#endif
/*}}}*/

/*{{{  #defines*/

#define  ALLOC_ARRAY	0x1
#define  ALLOC_SCALAR	0x2
#define  ALLOC_MAPVEC	0x4

#define  NULL_Alloc	((Alloc *) NULL)

/*}}}*/
/*{{{  typedef (Alloc)*/
typedef struct Alloc {
	int  alloc_type;	/* an 'ALLOC_...' flag */
	int  type;		/* datatype (a 'TY_...' flag) */
	int  num;		/* the number of this object in the stack */
	union {
		struct Expr  *ep;	/* for 'ALLOC_ARRAY' */
		int           intval;	/* for 'ALLOC_MAPVEC' */
	} len;
	bool  dead;		/* if TRUE the object can be de-allocated */
	struct Alloc  *next;
} Alloc;
/*}}}*/
/*{{{  externs*/

extern Alloc  *top_array_ap,  *top_scalar_ap;
extern int     n_temp_arrays,  n_temp_mapvecs,  n_temp_scalars;
/*}}}*/
/*{{{  'mapvec' macros*/
#ifdef V_ADAPT
/*----------------------------------------------------------------
Each 'mapvec' has the form:

INTEGER  ::  attrs,  a_rank,  e_rank,  array_struct_ptr,  proc_struct_ptr
INTEGER, DIMENSION (a_rank)  ::  l, n, s, segn, al, an, as,  tn,  b, fmt, d
INTEGER, DIMENSION (e_rank)  ::  ep, ed
INTEGER, DIMENSION (3 + 11*a_rank + 2*e_rank)  ::  mapvec

mapvec = (/ attrs,  array_struct_ptr,  proc_struct_ptr,         &
            l, n, s, segn, al, an, as, tn, b, fmt, d, ep, ed /)
----------------------------------------------------------------*/
#define  map_len(a_rank,e_rank)			(3 + 11*(a_rank) + 2*(e_rank))

/*------------------------*/
/* Subscripts in 'mapvec' */
/*------------------------*/
#define  map_subscr_l(a_rank,dim)		(3 + (dim))
#define  map_subscr_n(a_rank,dim)		(3 + (a_rank) + (dim))
#define  map_subscr_s(a_rank,dim)		(3 + 2*(a_rank) + (dim))
#define  map_subscr_segn(a_rank,dim)		(3 + 3*(a_rank) + (dim))
#define  map_subscr_al(a_rank,dim)		(3 + 4*(a_rank) + (dim))
#define  map_subscr_an(a_rank,dim)		(3 + 5*(a_rank) + (dim))
#define  map_subscr_as(a_rank,dim)		(3 + 6*(a_rank) + (dim))
#define  map_subscr_tn(a_rank,dim)		(3 + 7*(a_rank) + (dim))
#define  map_subscr_b(a_rank,dim)		(3 + 8*(a_rank) + (dim))
#define  map_subscr_fmt(a_rank,dim)		(3 + 9*(a_rank) + (dim))
#define  map_subscr_d(a_rank,dim)		(3 + 10*(a_rank) + (dim))

#define  map_subscr_ep(a_rank,e_rank,dim)	(3 + 11*(a_rank)+(dim))
#define  map_subscr_ed(a_rank,e_rank,dim)	(3 + 11*(a_rank)+(e_rank)+(dim))

#endif
/*}}}*/

#ifndef  V_ADAPT

#define  write_decls()			/* empty */

#endif

#endif
